@@ -20,19 +20,21 @@
 #include "quant.h"
 #include "bv.h"
 
+#include <cfloat>
+#include <algorithm>
 //#include <iostream>
-using namespace std;
+//using namespace std;
 
 
 /**
    @brief Static entry for regression case.
  */
-void Predict::Regression(double *_blockNumT, int *_blockFacT, unsigned int _nPredNum, unsigned int _nPredFac, std::vector<ForestNode> &_forestNode, std::vector<unsigned int> &_origin, std::vector<unsigned int> &_facOff, std::vector<unsigned int> &_facSplit, std::vector<unsigned int> &_leafOrigin, std::vector<LeafNode> &_leafNode, std::vector<BagRow> &_bagRow, std::vector<unsigned int> &_rank, std::vector<double> &yPred, unsigned int bagTrain) {
+void Predict::Regression(double *_blockNumT, int *_blockFacT, unsigned int _nPredNum, unsigned int _nPredFac, std::vector<ForestNode> &_forestNode, std::vector<unsigned int> &_origin, std::vector<unsigned int> &_facOff, std::vector<unsigned int> &_facSplit, std::vector<unsigned int> &_leafOrigin, std::vector<LeafNode> &_leafNode, std::vector<BagRow> &_bagRow, std::vector<unsigned int> &_rank, const std::vector<double> &yRanked, std::vector<double> &yPred, unsigned int bagTrain) {
   int nTree = _origin.size();
   unsigned int _nRow = yPred.size();
   PBPredict::Immutables(_blockNumT, _blockFacT, _nPredNum, _nPredFac, _nRow);
   LeafReg *leafReg = new LeafReg(_leafOrigin, _leafNode, _bagRow, _rank);
-  PredictReg *predictReg = new PredictReg(leafReg, nTree, _nRow, _leafNode.size());
+  PredictReg *predictReg = new PredictReg(leafReg, yRanked, nTree, _nRow, _leafNode.size());
   Forest *forest =  new Forest(_forestNode, _origin, _facOff, _facSplit, predictReg);
   BitMatrix *bag = leafReg->ForestBag(bagTrain);
   predictReg->PredictAcross(forest, yPred, bag);
@@ -53,10 +55,10 @@ void Predict::Quantiles(double *_blockNumT, int *_blockFacT, unsigned int _nPred
   unsigned int _nRow = yPred.size();
   PBPredict::Immutables(_blockNumT, _blockFacT, _nPredNum, _nPredFac, _nRow);
   LeafReg *leafReg = new LeafReg(_leafOrigin, _leafNode, _bagRow, _rank);
-  PredictReg *predictReg = new PredictReg(leafReg, nTree, _nRow, _leafNode.size());
+  PredictReg *predictReg = new PredictReg(leafReg, yRanked, nTree, _nRow, _leafNode.size());
   Forest *forest =  new Forest(_forestNode, _origin, _facOff, _facSplit, predictReg);
   BitMatrix *bag = leafReg->ForestBag(bagTrain);
-  Quant *quant = new Quant(predictReg, leafReg, yRanked, quantVec, qBin);
+  Quant *quant = new Quant(predictReg, leafReg, quantVec, qBin);
   predictReg->PredictAcross(forest, yPred, quant, &qPred[0], bag);
 
   delete bag;
@@ -90,11 +92,79 @@ void Predict::Classification(double *_blockNumT, int *_blockFacT, unsigned int _
 }
 
 
-PredictCtg::PredictCtg(const LeafCtg *_leafCtg, int _nTree, unsigned int _nRow, unsigned int _nonLeafIdx) : Predict(_nTree, _nRow, _nonLeafIdx), leafCtg(_leafCtg), ctgWidth(leafCtg->CtgWidth()) {
+PredictCtg::PredictCtg(const LeafCtg *_leafCtg, int _nTree, unsigned int _nRow, unsigned int _nonLeafIdx) : Predict(_nTree, _nRow, _nonLeafIdx), leafCtg(_leafCtg), ctgWidth(leafCtg->CtgWidth()), defaultScore(ctgWidth), defaultWeight(new double[ctgWidth]) {
+  for (unsigned int ctg = 0; ctg < ctgWidth; ctg++) {
+    defaultWeight[ctg] = -1.0;
+  }
 }
 
 
-PredictReg::PredictReg(const LeafReg *_leafReg, int _nTree, unsigned int _nRow, unsigned int _nonLeafIdx) : Predict(_nTree, _nRow, _nonLeafIdx), leafReg(_leafReg) {
+PredictReg::PredictReg(const LeafReg *_leafReg, const std::vector<double> &_yRanked, int _nTree, unsigned int _nRow, unsigned int _nonLeafIdx) : Predict(_nTree, _nRow, _nonLeafIdx), leafReg(_leafReg), yRanked(_yRanked), defaultScore(-DBL_MAX) {
+}
+
+
+/**
+   @brief Lazily sets default score.
+   TODO:  Ensure error if called when no bag present.
+
+   @return default forest score:  mean training response.
+ */
+double PredictReg::DefaultScore() {
+  if (defaultScore == -DBL_MAX) {
+    double sum = 0.0;
+    for (unsigned int i = 0; i < yRanked.size(); i++) {
+      sum += yRanked[i];
+    }
+    defaultScore = sum / yRanked.size();
+  }
+
+  return defaultScore;
+}
+
+
+/**
+   @brief Lazily sets default score.
+
+   @return default score.
+ */
+unsigned int PredictCtg::DefaultScore() {
+  if (defaultScore >= ctgWidth) {
+    (void) DefaultWeight(0);
+
+    defaultScore = 0;
+    double weightMax = defaultWeight[0];
+    for (unsigned int ctg = 1; ctg < ctgWidth; ctg++) {
+      if (defaultWeight[ctg] > weightMax) {
+	defaultScore = ctg;
+	weightMax = defaultWeight[ctg];
+      }
+    }
+  }
+
+  return defaultScore;
+}
+
+
+/**
+   @brief Lazily sets default weight.
+   TODO:  Ensure error if called when no bag present.
+
+   @return void.
+ */
+double PredictCtg::DefaultWeight(double *weightPredict) {
+  if (defaultWeight[0] < 0.0) {
+    leafCtg->ForestWeight(defaultWeight);
+  }
+
+  double rowSum = 0.0;
+  if (weightPredict != 0) {
+    for (unsigned int ctg = 0; ctg < ctgWidth; ctg++) {
+      weightPredict[ctg] = defaultWeight[ctg];
+      rowSum += weightPredict[ctg];
+    }
+  }
+
+  return rowSum;
 }
 
 
@@ -108,12 +178,17 @@ Predict::~Predict() {
 }
 
 
+PredictCtg::~PredictCtg() {
+  delete [] defaultWeight;
+}
+
+
 void PredictCtg::PredictAcross(const Forest *forest, const BitMatrix *bag, int *census, std::vector<int> &yPred, const std::vector<unsigned int> &yTest, int *conf, std::vector<double> &error, double *prob) {
   double *votes = new double[nRow * ctgWidth];
   for (unsigned int i = 0; i < nRow * ctgWidth; i++)
     votes[i] = 0;
   for (unsigned int rowStart = 0; rowStart < nRow; rowStart += rowBlock) {
-    unsigned int rowEnd = min(rowStart + rowBlock, nRow);
+    unsigned int rowEnd = std::min(rowStart + rowBlock, nRow);
     forest->PredictAcross(rowStart, rowEnd, bag);
     Score(votes, rowStart, rowEnd);
     if (prob != 0)
@@ -172,12 +247,12 @@ void PredictCtg::Validate(const std::vector<unsigned int> &yTest, const int yPre
    @return void, with output reference vector.
 */
 void PredictCtg::Vote(double *votes, int census[], int yPred[]) {
-  unsigned int row;
+  int row;
 
 #pragma omp parallel default(shared) private(row)
   {
 #pragma omp for schedule(dynamic, 1)
-  for (row = 0; row < nRow; row++) {
+  for (row = 0; row < int(nRow); row++) {
     int argMax = -1;
     double scoreMax = 0.0;
     double *score = votes + row * ctgWidth;
@@ -201,22 +276,30 @@ void PredictCtg::Vote(double *votes, int census[], int yPred[]) {
    @return internal vote table, with output reference vector.
  */
 void PredictCtg::Score(double *votes, unsigned int rowStart, unsigned int rowEnd) {
-  unsigned int blockRow;
+  int blockRow;
 
 // TODO:  Recast loop by blocks, to avoid
 // false sharing.
 #pragma omp parallel default(shared) private(blockRow)
   {
 #pragma omp for schedule(dynamic, 1)
-  for (blockRow = 0; blockRow < rowEnd - rowStart; blockRow++) {
+  for (blockRow = 0; blockRow < int(rowEnd - rowStart); blockRow++) {
     double *prediction = votes + (rowStart + blockRow) * ctgWidth;
+    unsigned int treesSeen = 0;
     for (int tc = 0; tc < nTree; tc++) {
       if (!IsBagged(blockRow, tc)) {
+	treesSeen++;
 	double val = leafCtg->GetScore(tc, LeafIdx(blockRow, tc));
 	unsigned int ctg = val; // Truncates jittered score for indexing.
 	prediction[ctg] += 1 + val - ctg;
       }
     }
+    if (treesSeen == 0) {
+      for (unsigned int ctg = 0; ctg < ctgWidth; ctg++) {
+	prediction[ctg] = 0.0;
+      }
+      prediction[DefaultScore()] = 1;
+    }
   }
   }
 }
@@ -226,8 +309,10 @@ void PredictCtg::Prob(double *prob, unsigned int rowStart, unsigned int rowEnd)
   for (unsigned int blockRow = 0; blockRow < rowEnd - rowStart; blockRow++) {
     double *probRow = prob + (rowStart + blockRow) * ctgWidth;
     double rowSum = 0.0;
+    unsigned int treesSeen = 0;
     for (int tc = 0; tc < nTree; tc++) {
       if (!IsBagged(blockRow, tc)) {
+	treesSeen++;
 	for (unsigned int ctg = 0; ctg < ctgWidth; ctg++) {
 	  double idxWeight = leafCtg->WeightCtg(tc, LeafIdx(blockRow, tc), ctg);
 	  probRow[ctg] += idxWeight;
@@ -235,6 +320,10 @@ void PredictCtg::Prob(double *prob, unsigned int rowStart, unsigned int rowEnd)
 	}
       }
     }
+    if (treesSeen == 0) {
+      rowSum = DefaultWeight(probRow);
+    }
+
     double recipSum = 1.0 / rowSum;
     for (unsigned int ctg = 0; ctg < ctgWidth; ctg++)
       probRow[ctg] *= recipSum;
@@ -246,7 +335,7 @@ void PredictCtg::Prob(double *prob, unsigned int rowStart, unsigned int rowEnd)
  */
 void PredictReg::PredictAcross(const Forest *forest, std::vector<double> &yPred, const BitMatrix *bag) {
   for (unsigned int rowStart = 0; rowStart < nRow; rowStart += rowBlock) {
-    unsigned int rowEnd = min(rowStart + rowBlock, nRow);
+    unsigned int rowEnd = std::min(rowStart + rowBlock, nRow);
     forest->PredictAcross(rowStart, rowEnd, bag);
     Score(rowStart, rowEnd, &yPred[rowStart]);
   }
@@ -260,7 +349,7 @@ void PredictReg::PredictAcross(const Forest *forest, std::vector<double> &yPred,
  */
 void PredictReg::PredictAcross(const Forest *forest, std::vector<double> &yPred, Quant *quant, double qPred[], const BitMatrix *bag) {
   for (unsigned int rowStart = 0; rowStart < nRow; rowStart += rowBlock) {
-    unsigned int rowEnd = min(rowStart + rowBlock, nRow);
+    unsigned int rowEnd = std::min(rowStart + rowBlock, nRow);
     forest->PredictAcross(rowStart, rowEnd, bag);
     Score(rowStart, rowEnd, &yPred[rowStart]);
     quant->PredictAcross(rowStart, rowEnd, qPred);
@@ -277,12 +366,12 @@ void PredictReg::PredictAcross(const Forest *forest, std::vector<double> &yPred,
   @return void, with output refererence vector.
  */
 void PredictReg::Score(unsigned int rowStart, unsigned int rowEnd, double yPred[]) {
-  unsigned int blockRow;
+  int blockRow;
 
 #pragma omp parallel default(shared) private(blockRow)
   {
 #pragma omp for schedule(dynamic, 1)
-    for (blockRow = 0; blockRow < rowEnd - rowStart; blockRow++) {
+  for (blockRow = 0; blockRow < int(rowEnd - rowStart); blockRow++) {
       double score = 0.0;
       int treesSeen = 0;
       for (int tc = 0; tc < nTree; tc++) {
@@ -291,8 +380,7 @@ void PredictReg::Score(unsigned int rowStart, unsigned int rowEnd, double yPred[
           score += leafReg->GetScore(tc, LeafIdx(blockRow, tc));
         }
       }
-      // Will be NA if row not oob at least once:
-      yPred[blockRow] = score / treesSeen;
+      yPred[blockRow] = treesSeen > 0 ? score / treesSeen : DefaultScore();
     }
   }
 }
