@@ -28,9 +28,15 @@
 #ifndef ARBORIST_RCPP_PREDBLOCK_H
 #define ARBORIST_RCPP_PREDBLOCK_H
 
+#include <Rcpp.h>
 using namespace Rcpp;
 
-void PredblockUnwrap(SEXP sPredBlock, int &_nRow, int &_nPredNum, int &_nPredFac, NumericMatrix &_blockNum, IntegerMatrix &_blockFac);
-void SignatureUnwrap(SEXP sSignature, IntegerVector &_predMap, List &_level);
+class RcppPredblock {
+ public:
+  static void Unwrap(SEXP sPredBlock, unsigned int &_nRow, unsigned int &_nPredNum, unsigned int &_nPredFac, NumericMatrix &_blockNum, IntegerMatrix &_blockFac);
+  static void SignatureUnwrap(SEXP sSignature, IntegerVector &_predMap, List &_level);
+  static void FactorRemap(IntegerMatrix &xFac, List &level, List &levelTrain);
+};
+
 
 #endif
