@@ -24,19 +24,19 @@
 #include "samplepred.h"
 #include "splitsig.h"
 #include "response.h"
-#include "restage.h"
 #include "splitpred.h"
-#include "run.h"
 #include "leaf.h"
 
+#include <algorithm>
 // Testing only:
 //#include <iostream>
-using namespace std;
+//using namespace std;
 
-int Train::trainBlock = 0;
+unsigned int Train::trainBlock = 0;
 unsigned int Train::nTree = 0;
 unsigned int Train::nRow = 0;
-int Train::nPred = 0;
+unsigned int Train::nPred = 0;
+
 
 /**
    @brief Initializes immutable values for "top-level" classes directly,
@@ -52,20 +52,18 @@ int Train::nPred = 0;
 
    @return void.
 */
-void Train::Init(double *_feNum, int _facCard[], int _cardMax, int _nPredNum, int _nPredFac, int _nRow, int _nTree, int _nSamp, double _feSampleWeight[], bool _withRepl, int _trainBlock, int _minNode, double _minRatio, int _totLevels, int _ctgWidth, int _predFixed, double _predProb[], double _regMono[]) {
+void Train::Init(const double _feNum[], const unsigned int _feCard[], unsigned int _cardMax, unsigned int _nPredNum, unsigned int _nPredFac, unsigned int _nRow, unsigned int _nTree, unsigned int _nSamp, const double _feSampleWeight[], bool _withRepl, unsigned int _trainBlock, unsigned int _minNode, double _minRatio, unsigned int _totLevels, unsigned int _ctgWidth, unsigned int _predFixed, const double _predProb[], const double _regMono[]) {
   nTree = _nTree;
   nRow = _nRow;
   nPred = _nPredNum + _nPredFac;
   trainBlock = _trainBlock;
-  PBTrain::Immutables(_feNum, _facCard, _cardMax, _nPredNum, _nPredFac, nRow);
+  PBTrain::Immutables(_feNum, _feCard, _cardMax, _nPredNum, _nPredFac, nRow);
   Sample::Immutables(nRow, nPred, _nSamp, _feSampleWeight, _withRepl, _ctgWidth, nTree);
   SPNode::Immutables(_ctgWidth);
   SplitSig::Immutables(nPred, _minRatio);
   Index::Immutables(_minNode, _totLevels);
   PreTree::Immutables(nPred, _nSamp, _minNode);
-  RestageMap::Immutables(nPred);
   SplitPred::Immutables(nPred, _ctgWidth, _predFixed, _predProb, _regMono);
-  Run::Immutables(nPred, _ctgWidth);
 }
 
 
@@ -83,14 +81,13 @@ void Train::DeImmutables() {
   Sample::DeImmutables();
   SPNode::DeImmutables();
   SplitPred::DeImmutables();
-  Run::DeImmutables();
 }
 
 
 /**
    @brief Regression constructor.
  */
-Train::Train(const std::vector<double> &_y, const std::vector<unsigned int> &_row2Rank, std::vector<unsigned int> &_origin, std::vector<unsigned int> &_facOrigin, double _predInfo[], std::vector<ForestNode> &_forestNode, std::vector<unsigned int> &_facSplit, std::vector<unsigned int> &_leafOrigin, std::vector<LeafNode> &_leafNode, std::vector<BagRow> &_bagRow, std::vector<unsigned int> &_rank) : forest(new Forest(_forestNode, _origin, _facOrigin, _facSplit)), predInfo(_predInfo), response(Response::FactoryReg(_y, _row2Rank, _leafOrigin, _leafNode, _bagRow, _rank)) {
+Train::Train(const std::vector<double> &_y, const std::vector<unsigned int> &_row2Rank, std::vector<unsigned int> &_origin, std::vector<unsigned int> &_facOrigin, double _predInfo[], std::vector<class ForestNode> &_forestNode, std::vector<unsigned int> &_facSplit, std::vector<unsigned int> &_leafOrigin, std::vector<class LeafNode> &_leafNode, std::vector<class BagRow> &_bagRow, std::vector<unsigned int> &_rank) : forest(new Forest(_forestNode, _origin, _facOrigin, _facSplit)), predInfo(_predInfo), response(Response::FactoryReg(_y, _row2Rank, _leafOrigin, _leafNode, _bagRow, _rank)) {
 }
 
 
@@ -105,7 +102,7 @@ Train::Train(const std::vector<double> &_y, const std::vector<unsigned int> &_ro
 
    @return forest height, with output reference parameter.
 */
-void Train::Regression(int _feRow[], int _feRank[], int _feInvNum[], const std::vector<double> &_y, const std::vector<unsigned int> &_row2Rank, std::vector<unsigned int> &_origin, std::vector<unsigned int> &_facOrigin, double _predInfo[], std::vector<ForestNode> &_forestNode, std::vector<unsigned int> &_facSplit, std::vector<unsigned int> &_leafOrigin, std::vector<LeafNode> &_leafNode, std::vector<BagRow> &_bagRow, std::vector<unsigned int> &_rank) {
+void Train::Regression(unsigned int _feRow[], unsigned int _feRank[], unsigned int _feInvNum[], const std::vector<double> &_y, const std::vector<unsigned int> &_row2Rank, std::vector<unsigned int> &_origin, std::vector<unsigned int> &_facOrigin, double _predInfo[], std::vector<class ForestNode> &_forestNode, std::vector<unsigned int> &_facSplit, std::vector<unsigned int> &_leafOrigin, std::vector<class LeafNode> &_leafNode, std::vector<class BagRow> &_bagRow, std::vector<unsigned int> &_rank) {
   Train *train = new Train(_y, _row2Rank, _origin, _facOrigin, _predInfo, _forestNode, _facSplit, _leafOrigin, _leafNode, _bagRow, _rank);
 
   RowRank *rowRank = new RowRank(_feRow, _feRank, _feInvNum, nRow, nPred);
@@ -129,7 +126,7 @@ Train::Train(const std::vector<unsigned int> &_yCtg, unsigned int _ctgWidth, con
 
    @return void.
 */
-void Train::Classification(int _feRow[], int _feRank[], int _feInvNum[], const std::vector<unsigned int> &_yCtg, int _ctgWidth, const std::vector<double> &_yProxy, std::vector<unsigned int> &_origin, std::vector<unsigned int> &_facOrigin, double _predInfo[], std::vector<ForestNode> &_forestNode, std::vector<unsigned int> &_facSplit, std::vector<unsigned int> &_leafOrigin, std::vector<LeafNode> &_leafNode, std::vector<BagRow> &_bagRow, std::vector<double> &_weight) {
+void Train::Classification(unsigned int _feRow[], unsigned int _feRank[], unsigned int _feInvNum[], const std::vector<unsigned int>  &_yCtg, unsigned int _ctgWidth, const std::vector<double> &_yProxy, std::vector<unsigned int> &_origin, std::vector<unsigned int> &_facOrigin, double _predInfo[], std::vector<class ForestNode> &_forestNode, std::vector<unsigned int> &_facSplit, std::vector<unsigned int> &_leafOrigin, std::vector<class LeafNode> &_leafNode, std::vector<class BagRow> &_bagRow, std::vector<double> &_weight) {
   Train *train = new Train(_yCtg, _ctgWidth, _yProxy, _origin, _facOrigin, _predInfo, _forestNode, _facSplit, _leafOrigin, _leafNode, _bagRow, _weight);
 
   RowRank *rowRank = new RowRank(_feRow, _feRank, _feInvNum, nRow, nPred);
@@ -156,13 +153,13 @@ Train::~Train() {
 */
 void Train::ForestTrain(const RowRank *rowRank) {
   for (unsigned treeStart = 0; treeStart < nTree; treeStart += trainBlock) {
-    unsigned int treeEnd = min(treeStart + trainBlock, nTree); // one beyond.
+    unsigned int treeEnd = std::min(treeStart + trainBlock, nTree); // one beyond.
     Block(rowRank, treeStart, treeEnd - treeStart);
   }
     
   // Normalizes 'predInfo' to per-tree means.
   double recipNTree = 1.0 / nTree;
-  for (int i = 0; i < nPred; i++)
+  for (unsigned int i = 0; i < nPred; i++)
     predInfo[i] *= recipNTree;
 
   forest->SplitUpdate(rowRank);
@@ -217,7 +214,7 @@ unsigned int Train::BlockPeek(PreTree **ptBlock, unsigned int tCount, unsigned i
   for (unsigned int i = 0; i < tCount; i++) {
     PreTree *pt = ptBlock[i];
     unsigned int height = pt->Height();
-    maxHeight = max(height, maxHeight);
+    maxHeight = std::max(height, maxHeight);
     blockHeight += height;
     blockFac += pt->BitWidth();
     blockLeaf += pt->LeafCount();
